#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-

"""Python Torrent Searcher/Scraper."""

from sys import path
from os.path import dirname
path.append(dirname(__file__))

import logging
from utils import ColorizeFilter
ch = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s %(levelname)8s %(name)s | %(message)s')
ch.setFormatter(formatter)

logger = logging.getLogger('torrentSearch')
logger.addHandler(ch)
logger.setLevel(logging.ERROR)  # This toggles all the logging in your app
logger.addFilter(ColorizeFilter())



#!/usr/bin/env python3.6
# -*- coding: utf-8 -*-
# @Author: KevinMidboe
# @Date:   2017-11-01 15:57:23
# @Last Modified by:   KevinMidboe
# @Last Modified time: 2017-12-22 12:07:18

import re
import os
import shutil
import logging
import colored
import configparser

from datetime import datetime
from colored import stylize

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
SYMBOLS = {
   'customary'     : ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
   'customary_ext' : ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa',
      'zetta', 'iotta'),
   'iec'           : ('BiB', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'),
   'iec_ext'       : ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi',
      'zebi', 'yobi'),
}

__all__ = ('ColorizeFilter', )
logger = logging.getLogger('torrentSearch')

def getConfig():
  """
  Read path and get configuartion file with site settings
  :return: config settings read from 'config.ini'
  :rtype: configparser.ConfigParser
  """
  config = configparser.ConfigParser()
  user_config_dir = os.path.expanduser("~") + "/.config/torrentSearch"

  config_dir = os.path.join(user_config_dir, 'config.ini')
  if not os.path.isfile(config_dir):
    defaultConfig = os.path.join(BASE_DIR, 'default_config.ini')
    logger.error('Missing config! Moved default_config.ini to {}.\nOpen this file and set all varaibles!'.format(config_dir))
    os.makedirs(user_config_dir, exist_ok=True)
    shutil.copyfile(defaultConfig, config_dir)

  config.read(config_dir)
  return config

class ColorizeFilter(logging.Filter):
   """
   Class for setting specific colors to levels of severity for log output
   """

   color_by_level = {
      logging.DEBUG: 'yellow',
      logging.WARNING: 'red',
      logging.ERROR: 'red',
      logging.INFO: 'white'
   }

   def filter(self, record):
      record.raw_msg = record.msg
      color = self.color_by_level.get(record.levelno)
      if color:
         record.msg = stylize(record.msg, colored.fg(color))
      return True

def sanitize(string, ignore_characters=None, replace_characters=None):
   """
   Sanitize a string to strip special characters.

   :param str string: the string to sanitize.
   :param set ignore_characters: characters to ignore.
   :return: the sanitized string.
   :rtype: str

   """
   # only deal with strings
   if string is None:
      return
   
   replace_characters = replace_characters or ''

   ignore_characters = ignore_characters or set()

   characters = ignore_characters
   if characters:
      string = re.sub(r'[%s]' % re.escape(''.join(characters)), replace_characters, string)

   return string

def return_re_match(string, re_statement):
   """
   Helper function for checking if a string contains a given regex statement

   :param str string, str re_statement: string we want to check and regex string we 
      check for in string
   :return: sanitized string of the match we found
   :rtype: str
   """
   if string is None:
      return

   m = re.search(re_statement, string)
   # This is here for a edge case for piratebay torrent dates
   if 'Y-day' in m.group():
      return datetime.now().strftime('%m-%d %Y')
   return sanitize(m.group(), '\xa0', ' ')

def representsInteger(str):
   """
   Checks if a string only contains integers

   :param str str: string we want to check only has integers
   :return: if string only contains integers
   :rtype: boolean
   """
   try:
      int(str)
      return True
   except ValueError:
      return False

def deHumansize(s):
   """ 
   Attempts to guess the string format based on default symbols
   set and return the corresponding bytes as an integer.
   When unable to recognize the format ValueError is raised.

   :param str s: human file size that we want to convert
   :return: the guessed bytes in from the human file size
   :rtype: int
   """

   init = s
   num = ""
   while s and s[0:1].isdigit() or s[0:1] == '.':
      num += s[0]
      s = s[1:]
   num = float(num)
   letter = s.strip()
   for name, sset in SYMBOLS.items():
      if letter in sset:
         break
   else:
      if letter == 'k':
         # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
         sset = SYMBOLS['customary']
         letter = letter.upper()
      else:
         raise ValueError("can't interpret %r" % init)
   prefix = {sset[0]:1}
   for i, s in enumerate(sset[1:]):
      prefix[s] = 1 << (i+1)*10
   return int(num * prefix[letter])

def humansize(nbytes):
   """ 
   Translates a size in bytes to a human readable size format

   :param int nbytes: integer of size of torrent file
   :return: size in bytes in a human readable format
   :rtype: str
   """
   suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
   i = 0
   while nbytes >= 1024 and i < len(suffixes)-1:
      nbytes /= 1024.
      i += 1
   f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
   return '{} {}'.format(f, suffixes[i])




#!/usr/bin/env python3.6

import logging
import re

import xml.etree.ElementTree as ET
from xml.etree.ElementTree import fromstring

from http_utils import build_url, fetch_url
from torrent import Torrent
from utils import humansize, representsInteger

logger = logging.getLogger('torrentSearch')

class Jackett(object):
   """docstring for Jackett"""
   def __init__(self, apikey, host, path, limit, ssl):
      super(Jackett, self).__init__()
      self.apikey = apikey
      self.host = host
      self.path = path
      self.page_limit = limit
      self.ssl = ssl

   def get_apikey(self):
      logger.debug('Using api key: {}'.format(self.apikey))
      return self.apikey

   def get_path(self):
      return self.path

   def get_page_limit(self):
      logger.debug('Current page limit: {} pages'.format(self.page_limit))
      return self.page_limit

   def search(self, query):
      """
      Starts the call to getting result from our indexer
      :param jackett.Jackett self: object instance
      :param str query: query we want to search for
      :return: list of results we found from scraping jackett output based on query
      :rtype: list
      """
      baseUrl = 'http://' + self.host
      path = self.get_path().split('/')
      url_args = {
         'apikey': self.get_apikey(),
         'limit': self.get_page_limit(),
         'q': query
      }
      logger.debug('Url arguments for jackett search: {}'.format(url_args))

      url = build_url(self.ssl, baseUrl, path, url_args)
      url = url.replace('+', '%20')
      res = fetch_url(url)

      return self.parse_xml_for_torrents(res.read())


   def find_xml_attribute(self, xml_element, attr):
      """
      Finds a specific XML attribute given a element name
      :param jackett.Jackett self: object instance
      :param xml.etree.ElementTree.Element xml_element: the xml tree we want to search
      :param str attr: the attribute/element name we want to find in the xml tree
      :return: the value of the element fiven the attr/element name
      :rtype: str
      """
      value = xml_element.find(attr)
      if (value != None):
         logger.debug('Found attribute: {}'.format(attr))
         return value.text
      else:
         logger.warning('Could not find attribute: {}'.format(attr))
         return ''

   def parse_xml_for_torrents(self, raw_xml):
      """
      Finds a specific XML attribute given a element name
      :param jackett.Jackett self: object instance
      :param bytes raw_xml: the xml page returned by querying jackett
      :return: all the torrents we found in the xml page
      :rtype: list
      """
      tree = ET.fromstring(raw_xml)
      channel = tree.find('channel')
      results = []
      for child in channel.findall('item'):
         title = self.find_xml_attribute(child, 'title')
         date = self.find_xml_attribute(child, 'pubDate')
         magnet = self.find_xml_attribute(child, 'link')
         size = self.find_xml_attribute(child, 'size')
         files = self.find_xml_attribute(child, 'files')
         foundUploader = re.findall('-1? *\w*', title)
         if len(foundUploader) > 0:
            uploader = str(foundUploader[-1][1:])
         else:
            uploader = None
         seeders = 0
         peers = 0

         for elm in child.findall('{http://torznab.com/schemas/2015/feed}attr'):
            if elm.get('name') == 'seeders':
               seeders = elm.get('value')
            if elm.get('name') == 'peers':
               peers = elm.get('value')
            if size != '' and representsInteger(size):
               size = humansize(int(size))

         logger.debug('Found torrent with info: \n\ttitle: {}\n\tmagnet: {}\n\tsize: {}\n\tdate: {}\
            \n\tseeders: {}\n\tpeers: {}'.format(title, magnet, size, date, seeders, peers))
         torrent = Torrent(title, magnet=magnet, size=size, uploader=uploader, date=date, seed_count=seeders, leech_count=peers)
         results.append(torrent)

      return results




#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-

import json
import logging
import logging.config

from jackett import Jackett
from piratebay import Piratebay
from utils import ColorizeFilter, getConfig

from pprint import pprint

logger = logging.getLogger('torrentSearch')

def createJSONList(torrents):
   """
   Iterates over all torrent objects in torrents and gets all attributes which are appended to a list

   :param list torrents: integer of size of torrent file
   :return: List of torrents with all their info in a JSON format
   :rtype: str
   """
   jsonList = []
   for torrent in torrents:
      jsonList.append(torrent.get_all_attr())

   return json.dumps(jsonList)

# This should be done front_end!
# I.E. filtering like this should be done in another script
# and should be done with the shared standard for types.
# PS: Is it the right move to use a shared standard? What
# happens if it is no longer public?
def chooseCandidate(torrent_list):
   """
   Takes a list of torrents and filtes then on a given release type
   Returns: JSON list of torrents only matching release type
   """
   interesting_torrents = []
   match_release_type = ['bdremux', 'brremux', 'remux', 'bdrip', 'brrip', 'blu-ray', 'bluray', 'bdmv', 'bdr', 'bd5']

   for torrent in torrent_list:
      intersecting_release_types = set(torrent.find_release_type()) & set(match_release_type)

      size, _, size_id = torrent.size.partition(' ')
      if intersecting_release_types:
         interesting_torrents.append(torrent)

   return interesting_torrents


def searchTorrentSite(query, site='jackett', filter=None, print_result=False, config=None):
   """
   Selects site based on input and finds torrents for that site based on query

   :param configparser.ConfigParser config: integer of size of torrent filest
   :param str query: query to search search torrents for
   :param str site: the site we want to index/scrape
   :param boolean print_result: if the in results should be printed to terminal
   :return: json list with results
   :rtype: str
   """
   if config is None:
      config = getConfig()
   logger.debug('Searching for query {} at {}'.format(query, site))

   if site == 'piratebay':
      pirate = Piratebay(config['PIRATEBAY']['HOST'], config['PIRATEBAY']['PATH'],
         config['PIRATEBAY']['LIMIT'], config['PIRATEBAY']['SSL'])
      torrents_found = pirate.search(query)
   elif site == 'jackett':
      jackett = Jackett(config['JACKETT']['APIKEY'], config['JACKETT']['HOST'],
         config['JACKETT']['PATH'], config['JACKETT']['LIMIT'], config.getboolean('JACKETT', 'SSL'))
      torrents_found = jackett.search(query)

   if (filter):
      torrents_found = chooseCandidate(torrents_found)

   jsonList = createJSONList(torrents_found)

   if (print_result):
      print(jsonList)

   return jsonList



#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-

"""Torrent Search.

Usage:
   search.py <query> [-s <site>] [-f] [-p | --print] [--debug | --warning | --error]
   search.py (-h | --help)
   search.py --version

Options:
   -h --help     Show this screen
   -s [site]     Site to index [default: piratebay] (piratebay|jackett)
   -p --print    Print result to console
   -f            Filter response on release type
   --version     Show version
   --debug       Print all debug logs
   --warning     Print only logged warnings
   --error       Print error messages (Error/Warning)
"""

import sys
import logging
import logging.config
import signal

from docopt import docopt
from __init__ import __version__

from search import searchTorrentSite
from utils import getConfig

logger = logging.getLogger('torrentSearch')

def main():
   """
   Main function, call searchTorrentSite
   """
   signal.signal(signal.SIGINT, signal_handler)

   arguments = docopt(__doc__, version=__version__)

   if arguments['--debug']:
      logger.level = logging.DEBUG
   elif arguments['--warning']:
      logger.level = logging.WARNING
   elif arguments['--error']:
      logger.level = logging.ERROR

   logger.info('Torrent Searcher')
   logger.debug(arguments)

   # Fetch config
   config = getConfig()

   if arguments['-s'] in config['DEFAULT']['SITE_OPTIONS'].split(','):
      site = arguments['-s']
      logger.debug('site selected: {}'.format(site))
   else:
      logger.error('"{}" is a invalid site. Select from: {}'.format(arguments['-s'], config['DEFAULT']['SITE_OPTIONS']))
      sys.exit()

   searchTorrentSite(arguments['<query>'], site, arguments['-f'], arguments['--print'], config)

def signal_handler(signal, frame):
   """
   Handle exit by Keyboardinterrupt
   """
   logger.info('\nGood bye!')
   sys.exit(0)

if __name__ == '__main__':
   main()



#!/usr/bin/env python3.6

from json import dumps
from utils import humansize, representsInteger

RELEASE_TYPES = ('bdremux', 'brremux', 'remux',
	'bdrip', 'brrip', 'blu-ray', 'bluray', 'bdmv', 'bdr', 'bd5',
	'web-cap', 'webcap', 'web cap',
	'webrip', 'web rip', 'web-rip', 'web',
	'webdl', 'web dl', 'web-dl', 'hdrip',
	'dsr', 'dsrip', 'satrip', 'dthrip', 'dvbrip', 'hdtv', 'pdtv', 'tvrip', 'hdtvrip',
	'dvdr', 'dvd-full', 'full-rip', 'iso',
	'ts', 'hdts', 'hdts', 'telesync', 'pdvd', 'predvdrip',
	'camrip', 'cam')

class Torrent(object):
	def __init__(self, name, magnet=None, size=None, uploader=None, date=None,
		seed_count=None, leech_count=None, url=None):
		self.name  = name
		self.magnet = magnet
		self.size = size
		self.uploader = uploader
		self.date = date
		self.seed_count = seed_count
		self.leech_count = leech_count
		self.url = url

		if (size != '' and representsInteger(size)):
			self.human_size = humansize(int(size))

	def find_release_type(self):
		name = self.name.casefold()
		return [r_type for r_type in RELEASE_TYPES if r_type in name]

	def get_all_attr(self):
		return ({'name': self.name, 'magnet': self.magnet,'uploader': self.uploader,
			'size': self.size,'date': self.date,'seed': self.seed_count,
			'leech': self.leech_count, 'url': self.url, 'release_type': self.find_release_type()})

	def to_json(self):
		return dumps(self.get_all_attr())

	def __repr__(self):
		return '<%s [%r]>' % (self.__class__.__name__, self.name)


#!/usr/bin/env python3.6

import re
import logging
from bs4 import BeautifulSoup

from http_utils import convert_query_to_percent_encoded_octets, build_url, fetch_url
from utils import return_re_match, deHumansize
from torrent import Torrent

logger = logging.getLogger('torrentSearch')

class Piratebay(object):
   """docstring for Piratebay"""
   def __init__(self, host, path, limit, ssl):
      super(Piratebay, self).__init__()
      self.host = host
      self.path = path
      self.page_limit = limit
      self.ssl = ssl
      self.page = 0
      self.total_pages = -1

   def get_path(self):
      return self.path

   def get_page_limit(self):
      return self.page_limit

   def search(self, query):
      """
      Starts the call to getting result from our thepiratebay site
      :param piratebay.Piratebay self: object instance
      :param str query: query we want to search for 
      :return: list of results we found from scraping thepiratebay site based on query
      :rtype: list
      """
      search_query = convert_query_to_percent_encoded_octets(query)
      baseUrl = 'http://' + self.host
      
      path = [self.get_path(), search_query, str(self.page)]
      url = build_url(self.ssl, baseUrl, path)

      res = fetch_url(url)

      return self.parse_raw_page_for_torrents(res.read())

   def removeHeader(self, bs4_element):
      if ('header' in bs4_element['class']):
         return bs4_element.find_next('tr')

      return bs4_element

   def has_magnet(self, href):
      return href and re.compile('magnet').search(href)

   def parse_raw_page_for_torrents(self, content):
      soup = BeautifulSoup(content, 'html.parser')
      content_searchResult = soup.body.find(id='searchResult')

      if content_searchResult is None:
         logging.info('No torrents found for the search criteria.')
         return None
      
      listElements = content_searchResult.tr
      
      torrentWrapper = self.removeHeader(listElements)

      torrents_found = []
      for torrentElement in torrentWrapper.find_all_next('td'):
         if torrentElement.find_all("div", class_='detName'):

            name = torrentElement.find('a', class_='detLink').get_text()
            url = torrentElement.find('a', class_='detLink')['href']
            magnet = torrentElement.find(href=self.has_magnet)
            
            uploader = torrentElement.find('a', class_='detDesc')          

            if uploader is None:
               uploader = torrentElement.find('i')
               
            uploader = uploader.get_text()

            info_text = torrentElement.find('font', class_='detDesc').get_text()
            
            date = return_re_match(info_text, r"(\d+\-\d+\s\d+)|(Y\-day\s\d{2}\:\d{2})")
            size = return_re_match(info_text, r"(\d+(\.\d+)?\s[a-zA-Z]+)")
            byteSize = deHumansize(size)

            # COULD NOT FIND HREF!
            if (magnet is None):
               logger.warning('Could not find magnet for {}'.format(name))
               continue

            seed_and_leech = torrentElement.find_all_next(attrs={"align": "right"})
            seed = seed_and_leech[0].get_text()
            leech = seed_and_leech[1].get_text()

            torrent = Torrent(name, magnet['href'], byteSize, uploader, date, seed, leech, url)

            torrents_found.append(torrent)
         else:
            logger.warning('Could not find torrent element on thepiratebay webpage.')
            continue

      logging.info('Found %s torrents for given search criteria.' % len(torrents_found))
      return torrents_found



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-

import os
from setuptools import setup, find_packages
from sys import path
from os.path import dirname

from __version__ import __version__

with open("README.md", "r", encoding="utf-8") as fh:
  long_description = fh.read()

print(find_packages())

setup(
    name='torrentSearch',
    version=__version__,
    packages=find_packages(),
    package_data={
        'torrentSearch': ['default_config.ini', '__version__.py'],
    },
    author='KevinMidboe',
    description='Search For Torrents',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        'bs4',
        'docopt',
	    'colored'
    ],
    url='https://github.com/KevinMidboe/torrent_search',
    classifiers=[
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
    ],
    entry_points={
        'console_scripts': [
            'torrentsearch = torrentSearch.__main__:main',
        ],
    },
)



#!/usr/bin/env python3.6

import logging
import sys

from urllib import parse, request
from urllib.error import URLError

logger = logging.getLogger('torrentSearch')

def build_url(ssl, baseUrl, path, args_dict={}):
   """
   Given the parameters joins them together to a url to 
   :param bool ssl: if ssl is to be used or not
   :param str baseUrl: the start of the url (http://thepiratebay.org)
   :param list path: the rest of the path to the url (['search', 'lucifer', '0'])
   :param dict args_dict: a dict with the query element we want to append to the url
   :return: complete url based on the inputs
   :rtype: str
   """
   url_parts = list(parse.urlparse(baseUrl))
   url_parts[0] = 'https' if ssl else 'http'
   url_parts[2] = '/'.join(path)
   url_parts[4] = parse.urlencode(args_dict)
   return parse.urlunparse(url_parts)

def convert_query_to_percent_encoded_octets(input_query):
   """
   Converts a string with spaces to a string separated by '%20'
   :param str input_query:
   :return: string with spaces replaced with '%20' if found any
   :rtype: str
   """
   if type(input_query) is list:
      input_query = ' '.join(input_query)

   return parse.quote(input_query)

def fetch_url(url):
   """
   Call and get output for a given url
   :param str url: the url we want to make a request to
   :return: a response object with contents and status code of the request
   :rtype: http.client.HTTPResponse
   """
   logger.debug('Fetching query: {}'.format(url))
   req = request.Request(url, headers={'User-Agent': 'Mozilla/5.0'})
   try:
       response = request.urlopen(req, timeout=60)
       return response
   except URLError as e:
      if hasattr(e, 'reason'):
         logger.error('We failed to reach a server with request: %s' % req.full_url)
         logger.error('Reason: %s' % e.reason)
      elif hasattr(e, 'code'):
         logger.error('The server couldn\'t fulfill the request.')
         logger.error('Error code: ', e.code)
      sys.exit()
